@@ -126,6 +126,8 @@ struct DocumentViewPrivate
                          q, SIGNAL(nextImageRequested()));
         QObject::connect(adapter, SIGNAL(toggleFullScreenRequested()),
                          q, SIGNAL(toggleFullScreenRequested()));
+        QObject::connect(adapter, SIGNAL(completed()),
+                         q, SLOT(slotCompleted()));
 
         adapter->loadConfig();
 
@@ -377,6 +379,9 @@ void DocumentView::createAdapterForDocument()
 void DocumentView::openUrl(const KUrl& url, const DocumentView::Setup& setup)
 {
     if (d->mDocument) {
+        if (url == d->mDocument->url()) {
+            return;
+        }
         disconnect(d->mDocument.data(), 0, this, 0);
     }
     d->mSetup = setup;
@@ -392,7 +397,7 @@ void DocumentView::openUrl(const KUrl& url, const DocumentView::Setup& setup)
         connect(d->mDocument.data(), SIGNAL(kindDetermined(KUrl)),
                 SLOT(finishOpenUrl()));
     } else {
-        finishOpenUrl();
+        QMetaObject::invokeMethod(this, "finishOpenUrl", Qt::QueuedConnection);
     }
     d->setupBirdEyeView();
 }
@@ -409,8 +414,6 @@ void DocumentView::finishOpenUrl()
     }
     createAdapterForDocument();
 
-    connect(d->mAdapter.data(), SIGNAL(completed()),
-            SLOT(slotCompleted()));
     connect(d->mDocument.data(), SIGNAL(loadingFailed(KUrl)),
             SLOT(slotLoadingFailed()));
     d->mAdapter->setDocument(d->mDocument);
